@@ -41,12 +41,8 @@ RESET_LOGGING = [
 
 RP_PREFIX = 'osc-placement-functional-tests-'
 
-# argparse in python 2 and 3 have different error messages
-ARGUMENTS_MISSING = 'too few arguments'
-ARGUMENTS_REQUIRED = 'argument %s is required'
-if six.PY3:
-    ARGUMENTS_MISSING = 'the following arguments are required'
-    ARGUMENTS_REQUIRED = 'the following arguments are required: %s'
+ARGUMENTS_MISSING = 'the following arguments are required'
+ARGUMENTS_REQUIRED = 'the following arguments are required: %s'
 
 
 class CommandException(Exception):
@@ -56,7 +52,7 @@ class CommandException(Exception):
 
 
 class BaseTestCase(base.BaseTestCase):
-    VERSION = None
+    VERSION = '1.0'
 
     def setUp(self):
         super(BaseTestCase, self).setUp()
@@ -204,7 +200,8 @@ class BaseTestCase(base.BaseTestCase):
 
     def resource_provider_list(self, uuid=None, name=None,
                                aggregate_uuids=None, resources=None,
-                               in_tree=None, required=None, forbidden=None):
+                               in_tree=None, required=None, forbidden=None,
+                               member_of=None, may_print_to_stderr=False):
         to_exec = 'resource provider list'
         if uuid:
             to_exec += ' --uuid ' + uuid
@@ -221,8 +218,12 @@ class BaseTestCase(base.BaseTestCase):
             to_exec += ' ' + ' '.join('--required %s' % t for t in required)
         if forbidden:
             to_exec += ' ' + ' '.join('--forbidden %s' % f for f in forbidden)
+        if member_of:
+            to_exec += ' ' + ' '.join(
+                ['--member-of %s' % m for m in member_of])
 
-        return self.openstack(to_exec, use_json=True)
+        return self.openstack(
+            to_exec, use_json=True, may_print_to_stderr=may_print_to_stderr)
 
     def resource_provider_delete(self, uuid):
         return self.openstack('resource provider delete ' + uuid)
@@ -244,8 +245,8 @@ class BaseTestCase(base.BaseTestCase):
             cmd += ' --project-id %s' % project_id
         if user_id:
             cmd += ' --user-id %s' % user_id
-        result = self.openstack(cmd, use_json=use_json,
-                                may_print_to_stderr=may_print_to_stderr)
+        result = self.openstack(
+            cmd, use_json=use_json, may_print_to_stderr=may_print_to_stderr)
 
         def cleanup(uuid):
             try:
@@ -259,19 +260,18 @@ class BaseTestCase(base.BaseTestCase):
         return result
 
     def resource_allocation_unset(self, consumer_uuid, provider=None,
-                                  use_json=True):
+                                  resource_class=None, use_json=True):
+        cmd = 'resource provider allocation unset %s' % consumer_uuid
+        if resource_class:
+            cmd += ' ' + ' '.join(
+                '--resource-class %s' % rc for rc in resource_class)
         if provider:
             # --provider can be specified multiple times so if we only get
             # a single string value convert to a list.
             if isinstance(provider, six.string_types):
                 provider = [provider]
-            cmd = 'resource provider allocation unset %s %s' % (
-                ' '.join('--provider %s' %
-                         rp_uuid for rp_uuid in provider),
-                consumer_uuid
-            )
-        else:
-            cmd = 'resource provider allocation unset %s' % consumer_uuid
+            cmd += ' ' + ' '.join(
+                '--provider %s' % rp_uuid for rp_uuid in provider)
         result = self.openstack(cmd, use_json=use_json)
 
         def cleanup(uuid):
@@ -289,15 +289,28 @@ class BaseTestCase(base.BaseTestCase):
         cmd = 'resource provider allocation delete ' + consumer_uuid
         return self.openstack(cmd)
 
-    def resource_inventory_show(self, uuid, resource_class):
-        cmd = 'resource provider inventory show {uuid} {rc}'.format(
-            uuid=uuid, rc=resource_class
+    def resource_inventory_show(
+        self, uuid, resource_class, *, include_used=False,
+    ):
+        resource = self.openstack(
+            f'resource provider inventory show {uuid} {resource_class}',
+            use_json=True,
         )
-        return self.openstack(cmd, use_json=True)
+        if not include_used:
+            del resource['used']
 
-    def resource_inventory_list(self, uuid):
-        return self.openstack('resource provider inventory list ' + uuid,
-                              use_json=True)
+        return resource
+
+    def resource_inventory_list(self, uuid, *, include_used=False):
+        resources = self.openstack(
+            f'resource provider inventory list {uuid}',
+            use_json=True,
+        )
+        if not include_used:
+            for resource in resources:
+                del resource['used']
+
+        return resources
 
     def resource_inventory_delete(self, uuid, resource_class=None):
         cmd = 'resource provider inventory delete {uuid}'.format(uuid=uuid)
@@ -408,18 +421,44 @@ class BaseTestCase(base.BaseTestCase):
         cmd = 'resource provider trait delete %s ' % uuid
         self.openstack(cmd)
 
-    def allocation_candidate_list(self, resources, required=None,
+    def allocation_candidate_list(self, resources=None, required=None,
                                   forbidden=None, limit=None,
-                                  aggregate_uuids=None):
-        cmd = 'allocation candidate list ' + ' '.join(
-            '--resource %s' % resource for resource in resources)
-        if required is not None:
-            cmd += ''.join([' --required %s' % t for t in required])
-        if forbidden:
-            cmd += ' ' + ' '.join('--forbidden %s' % f for f in forbidden)
+                                  aggregate_uuids=None, member_of=None,
+                                  may_print_to_stderr=False):
+        cmd = 'allocation candidate list '
+        cmd += self._allocation_candidates_option(
+            resources, required, forbidden, aggregate_uuids, member_of)
         if limit is not None:
             cmd += ' --limit %d' % limit
-        if aggregate_uuids:
-            cmd += ' ' + ' '.join(
-                   '--aggregate-uuid %s' % a for a in aggregate_uuids)
+        return self.openstack(
+            cmd, use_json=True, may_print_to_stderr=may_print_to_stderr)
+
+    def allocation_candidate_granular(self, groups, group_policy=None,
+                                      limit=None):
+        cmd = 'allocation candidate list '
+        for suffix, req_group in groups.items():
+            cmd += ' --group %s' % suffix
+            cmd += self._allocation_candidates_option(**req_group)
+            if limit is not None:
+                cmd += ' --limit %d' % limit
+            if group_policy is not None:
+                cmd += ' --group-policy %s' % group_policy
         return self.openstack(cmd, use_json=True)
+
+    def _allocation_candidates_option(self, resources=None, required=None,
+                                      forbidden=None, aggregate_uuids=None,
+                                      member_of=None):
+        opt = ''
+        if resources:
+            opt += ' ' + ' '.join(
+                '--resource %s' % resource for resource in resources)
+        if required is not None:
+            opt += ''.join([' --required %s' % t for t in required])
+        if forbidden:
+            opt += ' ' + ' '.join('--forbidden %s' % f for f in forbidden)
+        if aggregate_uuids:
+            opt += ' ' + ' '.join(
+                '--aggregate-uuid %s' % a for a in aggregate_uuids)
+        if member_of:
+            opt += ' ' + ' '.join(['--member-of %s' % m for m in member_of])
+        return opt
